import { useState, useEffect, useContext, useRef } from "react";
import { useDebouncedCallback } from "use-debounce";
import { CanvasContext } from "./CanvasContext.js";
import { compareObjects } from "./utils.js";
export function useRoute(route, params, options = { subscribe: true }) {
    const { api } = useContext(CanvasContext);
    const [isLoading, setIsLoading] = useState(true);
    const [error, setError] = useState(null);
    const [data, setData] = useState(null);
    const subscribe = options.subscribe ?? true;
    const refetch = useDebouncedCallback((api, route, params) => {
        setIsLoading(true);
        api
            .getRoute(route, params)
            .then((results) => setData(results))
            .catch((err) => setError(err))
            .finally(() => setIsLoading(false));
    }, 500, { leading: true, trailing: true });
    const routeRef = useRef(null);
    const paramsRef = useRef({});
    useEffect(() => {
        if (api === null) {
            return;
        }
        if (route !== routeRef.current || !compareObjects(params, paramsRef.current)) {
            refetch(api, route, params);
        }
        routeRef.current = route;
        paramsRef.current = params;
        if (subscribe) {
            const listener = ({}) => refetch(api, route, params);
            api.addEventListener("update", listener);
            return () => api.removeEventListener("update", listener);
        }
    }, [api, route, params]);
    return { error, data, isLoading };
}
